/*
 * Read 6 PWM signals from an RC receiver.
 *
 * Channels 0-5 are on digital pins 2-7 (i.e. PD2-PD7 = PCINT18-23).
 */

#include <DRV8835MotorShield.h>
#include <Wire.h>
#define HIGH_PIN 11
DRV8835MotorShield motors;
int spd1;
int spd2;
int spd3;

int ch3;

#define DIR_PIN 4
#define PWM_PIN 5

// This always contains the last length measured on each channel.
volatile uint16_t last_pulse_lengths[6];

// Interrupt triggered on every change in pins 2-7.
ISR(PCINT2_vect)
{
    static uint8_t last_pin_states;
    static uint16_t start_times[6];  // start time for each channel
    uint8_t pin_states = PIND >> 2;  // shifted to bits 0-5
    uint16_t now = micros();

    // Check every changed bit of the port.
    for (int i = 0; i < 6; i++) {
        uint8_t mask = _BV(i);

        // Record start time on rising edge.
        if ((pin_states & mask) && !(last_pin_states & mask))
            start_times[i] = now;

        // Record pulse length on falling edge.
        if (!(pin_states & mask) && (last_pin_states & mask))
            last_pulse_lengths[i] = now - start_times[i];
    }
    last_pin_states = pin_states;
}

// Return the pulse lengths avoiding race conditions.
static void get_pulse_lengths(uint16_t lengths[6])
{
    for (int i = 0; i < 6; i++) {
        noInterrupts();
        uint16_t tmp = last_pulse_lengths[i];
        interrupts();
        lengths[i] = tmp;
    }
}

void setup()
{
    // Setup the pin change interrupt.
    PCIFR  = _BV(PCIF2);  // clear pin change interrupt flag
    PCICR  = _BV(PCIE2);  // enable pin change interrupt
    PCMSK2 = 0xfc;        // sense changes on pins 2-7 (i.e. PD2-PD7)

    pinMode(13, INPUT);
    pinMode(HIGH_PIN, OUTPUT);

    pinMode(DIR_PIN, OUTPUT);

    Serial.begin(9600);
}

void loop()
{
    // Report the pulse lengths on the serial console.
    uint16_t pulse_lengths[6];
    get_pulse_lengths(pulse_lengths);
    for (int i = 0; i < 6; i++) {
        Serial.print(pulse_lengths[i]);
        if (i < 5) Serial.print(", ");
        else Serial.println();
    }

    digitalWrite(HIGH_PIN, HIGH);
//    digitalWrite(DIR_PIN, HIGH);

    spd1 = map(pulse_lengths[0], 990,1950, -400, 400); //center over zero
    
    Serial.print("Speed 1:"); // Print the value of 
    Serial.println(spd1);        // each channel

    motors.setM1Speed(spd1);

    spd2 = map(pulse_lengths[1], 990,1950, -400, 400); //center over zero
    Serial.print("Speed 2:"); // Print the value of 
    Serial.println(spd2);        // each channel

    motors.setM2Speed(spd2);

// This uses pulseIn for the sprawl motor and single channel motor driver
    ch3 = pulseIn(13, HIGH, 25000);
    Serial.print("Channel 3:"); // Print the value of 
    Serial.println(ch3);        // each channel
  
    spd3 = map(ch3, 850,1720, -150, 150); //center over zero
    Serial.print("Speed 3:"); // Print the value of    
    Serial.println(spd3);        // each channel

// Set the direction of the motor of the single channel motor driver
    if (spd3 < 0) digitalWrite(DIR_PIN, HIGH);
    if (spd3 > 0) digitalWrite(DIR_PIN, LOW);

// Using a knob on the FS-i6 to control the sprawl. Don't want it changing if near zero, so filter for that case
// Also getting intermittent spikes to -2xx, so filtering that out also
    if (abs(spd3) < 40 || abs(spd3) > 150)
    {analogWrite(PWM_PIN, 0);}
    else
    {
      if (spd3>0)
      // Drop the motor speed when lowering sprawl height for easier positional accuracy.
      // Don't have to fight gravity and lift the mass of the robot in this direction.
      {analogWrite(PWM_PIN, 25);}
      else
      {analogWrite(PWM_PIN, abs(spd3));}
    }

    delay(500);
}
